@@ -15,6 +15,7 @@
 #include <sys/socket.h>
 #include <netdb.h>
 #include "perftest_communication.h"
+#include "host_memory.h"
 #ifdef HAVE_SRD
 #include <infiniband/efadv.h>
 #endif
@@ -243,6 +244,7 @@ static int send_qp_num_for_ah(struct pingpong_context *ctx,
 		ne = ibv_poll_cq(ctx->send_cq, 1,&wc);
 	} while (ne == 0);
 
+	//coverity[uninit_use]
 	if (wc.status || wc.opcode != IBV_WC_SEND || wc.wr_id != 0) {
 		fprintf(stderr, " Couldn't post send my QP number %d\n",(int)wc.status);
 		return 1;
@@ -267,6 +269,7 @@ static int create_ah_from_wc_recv(struct pingpong_context *ctx,
 		ne = ibv_poll_cq(ctx->recv_cq,1,&wc);
 	} while (ne == 0);
 
+	//coverity[uninit_use]
 	if (wc.status || !(wc.opcode & IBV_WC_RECV) || wc.wr_id != 0) {
 		fprintf(stderr, "Bad wc status when trying to create AH -- %d -- %d \n",(int)wc.status,(int)wc.wr_id);
 		return 1;
@@ -276,6 +279,7 @@ static int create_ah_from_wc_recv(struct pingpong_context *ctx,
 	user_param->ah_allocated = 1;
 	user_param->rem_ud_qpn = ntohl(wc.imm_data);
 	ibv_query_qp(ctx->qp[0],&attr, IBV_QP_QKEY,&init_attr);
+	//coverity[uninit_use]
 	user_param->rem_ud_qkey = attr.qkey;
 
 	return 0;
@@ -347,6 +351,7 @@ static int ethernet_read_keys(struct pingpong_dest *rem_dest,
 				(unsigned int*)&rem_dest->psn, &rem_dest->rkey,&rem_dest->vaddr,&rem_dest->srqn);
 
 		if (parsed != 7) {
+			//coverity[string_null]
 			fprintf(stderr, "Couldn't parse line <%.*s>\n",(int)sizeof msg, msg);
 			return 1;
 		}
@@ -479,6 +484,7 @@ static int rdma_write_keys(struct pingpong_dest *my_dest,
 		ne = ibv_poll_cq(comm->rdma_ctx->send_cq, 1,&wc);
 	} while (ne == 0);
 
+	//coverity[uninit_use]
 	if (wc.status || wc.opcode != IBV_WC_SEND || wc.wr_id != SYNC_SPEC_ID) {
 		fprintf(stderr, " Bad wc status %d\n",(int)wc.status);
 		return 1;
@@ -504,6 +510,7 @@ static int rdma_read_keys(struct pingpong_dest *rem_dest,
 	} while (ne == 0);
 
 	if (wc.status || !(wc.opcode & IBV_WC_RECV) || wc.wr_id != SYNC_SPEC_ID) {
+		//coverity[uninit_use_in_call]
 		fprintf(stderr, "Bad wc status -- %d -- %d \n",(int)wc.status,(int)wc.wr_id);
 		return 1;
 	}
@@ -626,6 +633,7 @@ static int get_best_gid_index (struct pingpong_context *ctx,
 			if (ibv_query_gid_ex(ctx->context, port, i, &roce_version_rival, 0))
 				continue;
 
+			//coverity[uninit_use_in_call]
 			if (check_better_roce_version(roce_version.gid_type, roce_version_rival.gid_type) == RIGHT_IS_BETTER) {
 				gid_index = i;
 			}
@@ -661,7 +669,7 @@ static int ethernet_client_connect(struct perftest_comm *comm)
 
 	int sockfd = -1;
 	memset(&hints, 0, sizeof hints);
-	hints.ai_family   = AF_INET;
+	hints.ai_family   = comm->rdma_params->ai_family;
 	hints.ai_socktype = SOCK_STREAM;
 
 	if (comm->rdma_params->has_source_ip) {
@@ -682,6 +690,7 @@ static int ethernet_client_connect(struct perftest_comm *comm)
 				if (bind(sockfd, (struct sockaddr *)&source, sizeof(source)) < 0)
 				{
 					fprintf(stderr, "Failed to bind socket\n");
+					close(sockfd);
 					return 1;
 				}
 			}
@@ -717,7 +726,7 @@ static int ethernet_server_connect(struct perftest_comm *comm)
 
 	memset(&hints, 0, sizeof hints);
 	hints.ai_flags    = AI_PASSIVE;
-	hints.ai_family   = AF_INET;
+	hints.ai_family   = comm->rdma_params->ai_family;
 	hints.ai_socktype = SOCK_STREAM;
 
 	if (check_add_port(&service,comm->rdma_params->port,src_ip,&hints,&res))
@@ -727,6 +736,9 @@ static int ethernet_server_connect(struct perftest_comm *comm)
 	}
 
 	for (t = res; t; t = t->ai_next) {
+		if (t->ai_family != comm->rdma_params->ai_family)
+			continue;
+
 		sockfd = socket(t->ai_family, t->ai_socktype, t->ai_protocol);
 
 		if (sockfd >= 0) {
@@ -792,6 +804,7 @@ int set_up_connection(struct pingpong_context *ctx,
 			if (ibv_query_gid(ctx->context, user_param->ib_port, user_param->gid_index, &temp_gid))
 				return -1;
 		} else {
+			//coverity[uninit_use_in_call]
 			user_param->gid_index = get_best_gid_index(ctx, user_param, &attr, user_param->ib_port);
 			if (user_param->gid_index < 0)
 				return -1;
@@ -833,8 +846,8 @@ int set_up_connection(struct pingpong_context *ctx,
 				my_dest[i].lid   = ctx_get_local_lid(ctx->context,user_param->ib_port2);
 				my_dest[i].gid_index = user_param->gid_index2;
 			}
-			/*single-port case*/
 		} else {
+			/*single-port case*/
 			my_dest[i].lid   = ctx_get_local_lid(ctx->context,user_param->ib_port);
 			my_dest[i].gid_index = user_param->gid_index;
 		}
@@ -855,21 +868,14 @@ int set_up_connection(struct pingpong_context *ctx,
 			if (i % num_of_qps < num_of_qps_per_port)
 				memcpy(my_dest[i].gid.raw,temp_gid.raw ,16);
 
-			else
+			else {
+				//coverity[uninit_use_in_call]
 				memcpy(my_dest[i].gid.raw,temp_gid2.raw ,16);
+			}
 		} else {
+			//coverity[uninit_use_in_call]
 			memcpy(my_dest[i].gid.raw,temp_gid.raw ,16);
 		}
-
-		/*
-		We do not fail test upon lid above RoCE.
-		if ( (user_param->gid_index < 0) ||  ((user_param->gid_index2 < 0) && (user_param->dualport == ON))  ){
-			if (!my_dest[i].lid) {
-				fprintf(stderr," Local lid 0x0 detected. Is an SM running? \n");
-				return -1;
-			}
-		}
-		*/
 	}
 
 	#ifdef HAVE_XRCD
@@ -902,8 +908,8 @@ int set_up_connection(struct pingpong_context *ctx,
 int rdma_client_connect(struct pingpong_context *ctx,struct perftest_parameters *user_param)
 {
 	char *service;
-	int temp,num_of_retry= NUM_OF_RETRIES;
-	struct sockaddr_in sin, source_sin;
+	int temp, num_of_retry = NUM_OF_RETRIES;
+	struct sockaddr_storage sin, source_sin;
 	struct sockaddr *source_ptr = NULL;
 	struct addrinfo *res;
 	struct rdma_cm_event *event;
@@ -911,7 +917,7 @@ int rdma_client_connect(struct pingpong_context *ctx,struct perftest_parameters
 	struct addrinfo hints;
 
 	memset(&hints, 0, sizeof hints);
-	hints.ai_family   = AF_INET;
+	hints.ai_family   = user_param->ai_family;
 	hints.ai_socktype = SOCK_STREAM;
 
 	if (check_add_port(&service,user_param->port,user_param->servername,&hints,&res)) {
@@ -919,11 +925,19 @@ int rdma_client_connect(struct pingpong_context *ctx,struct perftest_parameters
 		return FAILURE;
 	}
 
-	if (res->ai_family != PF_INET) {
+	if (res->ai_family != user_param->ai_family) {
+		freeaddrinfo(res);
 		return FAILURE;
 	}
-	memcpy(&sin, res->ai_addr, sizeof(sin));
-	sin.sin_port = htons((unsigned short)user_param->port);
+
+	if (res->ai_addr->sa_family == AF_INET) {
+		memcpy(&sin, res->ai_addr, sizeof(struct sockaddr_in));
+	}
+	else {
+		memcpy(&sin, res->ai_addr, sizeof(struct sockaddr_in6));
+	}
+	sockaddr_set_port((struct sockaddr *)&sin, (unsigned short)user_param->port);
+	freeaddrinfo(res);
 
 	if (user_param->has_source_ip) {
 		if (check_add_port(&service, 0x0, user_param->source_ip, &hints, &res))
@@ -932,10 +946,16 @@ int rdma_client_connect(struct pingpong_context *ctx,struct perftest_parameters
 			return FAILURE;
 		}
 		memset(&source_sin, 0x0, sizeof(source_sin));
-		memcpy(&source_sin, res->ai_addr, sizeof(source_sin));
+		//coverity[deref_after_free]
+		if (res->ai_addr->sa_family == AF_INET) {
+			memcpy(&source_sin, res->ai_addr, sizeof(struct sockaddr_in));
+		}
+		else {
+			memcpy(&source_sin, res->ai_addr, sizeof(struct sockaddr_in6));
+		}
 		source_ptr = (struct sockaddr *)&source_sin;
+		freeaddrinfo(res);
 	}
-
 	while (1)
 	{
 
@@ -954,6 +974,7 @@ int rdma_client_connect(struct pingpong_context *ctx,struct perftest_parameters
 			return FAILURE;
 		}
 
+		//coverity[uninit_use]
 		if (event->event == RDMA_CM_EVENT_ADDR_ERROR) {
 			num_of_retry--;
 			rdma_ack_cm_event(event);
@@ -1117,29 +1138,36 @@ int rdma_server_connect(struct pingpong_context *ctx,
 	struct rdma_conn_param conn_param;
 	struct addrinfo hints;
 	char *service;
-	struct sockaddr_in sin;
+	struct sockaddr_storage sin;
 	char* src_ip = user_param->has_source_ip ? user_param->source_ip : NULL;
 
 	memset(&hints, 0, sizeof hints);
 	hints.ai_flags    = AI_PASSIVE;
-	hints.ai_family   = AF_INET;
+	hints.ai_family   = user_param->ai_family;
 	hints.ai_socktype = SOCK_STREAM;
 
-        memset(&sin, 0x0, sizeof(sin));
-
 	if (check_add_port(&service,user_param->port,src_ip,&hints,&res))
 	{
 		fprintf(stderr, "Problem in resolving basic address and port\n");
 		return FAILURE;
 	}
 
-	if (res->ai_family != PF_INET) {
+	if (res->ai_family != user_param->ai_family) {
+		freeaddrinfo(res);
 		return FAILURE;
 	}
-	memcpy(&sin, res->ai_addr, sizeof(sin));
-	sin.sin_port = htons((unsigned short)user_param->port);
 
-	if (rdma_bind_addr(ctx->cm_id_control,(struct sockaddr *)&sin)) {
+	if (res->ai_addr->sa_family == AF_INET) {
+		memcpy(&sin, res->ai_addr, sizeof(struct sockaddr_in));
+	}
+	else {
+		memcpy(&sin, res->ai_addr, sizeof(struct sockaddr_in6));
+	}
+
+	sockaddr_set_port((struct sockaddr *)&sin, (unsigned short)user_param->port);
+	freeaddrinfo(res);
+
+	if (rdma_bind_addr(ctx->cm_id_control, (struct sockaddr *)&sin)) {
 		fprintf(stderr," rdma_bind_addr failed\n");
 		return 1;
 	}
@@ -1154,6 +1182,7 @@ int rdma_server_connect(struct pingpong_context *ctx,
 		return 1;
 	}
 
+	//coverity[uninit_use]
 	if (event->event != RDMA_CM_EVENT_CONNECT_REQUEST) {
 		fprintf(stderr, "bad event waiting for connect request %d\n",event->event);
 		return 1;
@@ -1220,7 +1249,6 @@ int rdma_server_connect(struct pingpong_context *ctx,
 
 	rdma_ack_cm_event(event);
 	rdma_destroy_id(ctx->cm_id_control);
-	freeaddrinfo(res);
 	return 0;
 }
 
@@ -1234,6 +1262,7 @@ int create_comm_struct(struct perftest_comm *comm,
 	memset(comm->rdma_params, 0, sizeof(struct perftest_parameters));
 
 	comm->rdma_params->port		   	= user_param->port;
+	comm->rdma_params->ai_family	   	= user_param->ai_family;
 	comm->rdma_params->sockfd      		= -1;
 	comm->rdma_params->gid_index   		= user_param->gid_index;
 	comm->rdma_params->gid_index2 		= user_param->gid_index2;
@@ -1256,6 +1285,8 @@ int create_comm_struct(struct perftest_comm *comm,
 	comm->rdma_params->use_old_post_send	= user_param->use_old_post_send;
 	comm->rdma_params->source_ip		= user_param->source_ip;
 	comm->rdma_params->has_source_ip	= user_param->has_source_ip;
+	comm->rdma_params->memory_type		= MEMORY_HOST;
+	comm->rdma_params->memory_create	= host_memory_create;
 
 	if (user_param->use_rdma_cm) {
 
@@ -1270,7 +1301,11 @@ int create_comm_struct(struct perftest_comm *comm,
 		comm->rdma_params->size = sizeof(struct pingpong_dest);
 		comm->rdma_ctx->context = NULL;
 
-		MAIN_ALLOC(comm->rdma_ctx->mr, struct ibv_mr*, user_param->num_of_qps, free_rdma_ctx);
+		comm->rdma_ctx->memory = comm->rdma_params->memory_create(comm->rdma_params);
+		if (comm->rdma_ctx->memory == NULL)
+			goto free_rdma_ctx;
+
+		MAIN_ALLOC(comm->rdma_ctx->mr, struct ibv_mr*, user_param->num_of_qps, free_memory_ctx);
 		MAIN_ALLOC(comm->rdma_ctx->buf, void* , user_param->num_of_qps, free_mr);
 		MAIN_ALLOC(comm->rdma_ctx->qp,struct ibv_qp*,comm->rdma_params->num_of_qps, free_buf);
 		#ifdef HAVE_IBV_WR_API
@@ -1314,6 +1349,8 @@ free_buf:
 	free(comm->rdma_ctx->buf);
 free_mr:
 	free(comm->rdma_ctx->mr);
+free_memory_ctx:
+	comm->rdma_ctx->memory->destroy(comm->rdma_ctx->memory);
 free_rdma_ctx:
 	free(comm->rdma_ctx);
 free_rdma_params:
@@ -1338,6 +1375,11 @@ void dealloc_comm_struct(struct perftest_comm *comm,
 		#ifdef HAVE_DCS
 		free(comm->rdma_ctx->dci_stream_id);
 		#endif
+		if (comm->rdma_ctx->memory != NULL)
+		{
+			comm->rdma_ctx->memory->destroy(comm->rdma_ctx->memory);
+			comm->rdma_ctx->memory = NULL;
+		}
 		free(comm->rdma_ctx);
 	}
 
@@ -1508,6 +1550,7 @@ int rdma_read_data(void *data,
 	} while (ne == 0);
 
 	if (wc.status || !(wc.opcode & IBV_WC_RECV) || wc.wr_id != SYNC_SPEC_ID) {
+		//coverity[uninit_use_in_call]
 		fprintf(stderr, "Bad wc status -- %d -- %d \n",(int)wc.status,(int)wc.wr_id);
 		return 1;
 	}
@@ -1555,6 +1598,7 @@ int rdma_write_data(void *data,
 		ne = ibv_poll_cq(comm->rdma_ctx->send_cq, 1,&wc);
 	} while (ne == 0);
 
+	//coverity[uninit_use]
 	if (wc.status || wc.opcode != IBV_WC_SEND || wc.wr_id != SYNC_SPEC_ID) {
 		fprintf(stderr, " Bad wc status %d\n",(int)wc.status);
 		return 1;
@@ -1713,7 +1757,7 @@ void ctx_print_pingpong_data(struct pingpong_dest *element,
 	int is_there_mgid,local_mgid,remote_mgid;
 
 	/* use dlid value from user (if user specified and only on the remote side) */
-	uint16_t dlid = (comm->rdma_params->dlid && comm->rdma_params->side) ?
+	uint16_t dlid = (comm->rdma_params->dlid && comm->rdma_params->side == REMOTE) ?
 				comm->rdma_params->dlid : element->lid;
 
 	if (comm->rdma_params->output != FULL_VERBOSITY)
@@ -1722,8 +1766,8 @@ void ctx_print_pingpong_data(struct pingpong_dest *element,
 	printf(BASIC_ADDR_FMT, sideArray[comm->rdma_params->side], dlid, element->qpn, element->psn);
 
 	switch (comm->rdma_params->verb) {
-		case 2  : printf(READ_FMT,element->out_reads);
-		case 1  : printf(RDMA_FMT,element->rkey,element->vaddr);
+		case READ  : printf(READ_FMT,element->out_reads);
+		case WRITE : printf(RDMA_FMT,element->rkey,element->vaddr);
 		default : ;
 	}
 
@@ -1735,8 +1779,8 @@ void ctx_print_pingpong_data(struct pingpong_dest *element,
 
 	putchar('\n');
 
-	local_mgid    = (comm->rdma_params->side == 0)  && (comm->rdma_params->machine == 0);
-	remote_mgid   = (comm->rdma_params->side == 1)  && (comm->rdma_params->machine == 1);
+	local_mgid    = (comm->rdma_params->side == LOCAL)  && (comm->rdma_params->machine == SERVER);
+	remote_mgid   = (comm->rdma_params->side == REMOTE)  && (comm->rdma_params->machine == CLIENT);
 	is_there_mgid =  comm->rdma_params->duplex || remote_mgid || local_mgid;
 
 	if ((comm->rdma_params->gid_index > -1 || (comm->rdma_params->use_mcg && is_there_mgid)) && comm->rdma_params->connection_type != RawEth) {
@@ -1915,6 +1959,7 @@ int check_mtu(struct ibv_context *context,struct perftest_parameters *user_param
 				exit(1);
 			}
 
+			//coverity[uninit_use]
 			if (user_param->size > port_attr.max_msg_sz) {
 				if (user_param->test_method == RUN_ALL || !user_param->req_size) {
 					fprintf(stderr, " Max msg size is %u\n",
@@ -1927,18 +1972,37 @@ int check_mtu(struct ibv_context *context,struct perftest_parameters *user_param
 					return FAILURE;
 				}
 			}
-		} else if (user_param->verb == READ) {
-#ifdef HAVE_SRD_WITH_RDMA_READ
-			struct efadv_device_attr efa_device_attr = {};
+		} else if (user_param->verb == READ || user_param->verb == WRITE) {
+#ifdef HAVE_SRD
+			struct efadv_device_attr efa_device_attr = {0};
 
 			if (efadv_query_device(context, &efa_device_attr, sizeof(efa_device_attr))) {
 				fprintf(stderr, " Error when trying to query EFA device\n");
 				exit(1);
 			}
-			if (!(efa_device_attr.device_caps & EFADV_DEVICE_ATTR_CAPS_RDMA_READ)) {
-				fprintf(stderr, "Read verb is not supported with this EFA device\n");
+
+			if (user_param->verb == READ) {
+#ifdef HAVE_SRD_WITH_RDMA_READ
+				if (!(efa_device_attr.device_caps & EFADV_DEVICE_ATTR_CAPS_RDMA_READ)) {
+					fprintf(stderr, "Read verb is not supported with this EFA device\n");
+					exit(1);
+				}
+#else
+				fprintf(stderr, "SRD connection not possible in READ verb\n");
 				exit(1);
+#endif
+			} else if (user_param->verb == WRITE) {
+#ifdef HAVE_SRD_WITH_RDMA_WRITE
+				if (!(efa_device_attr.device_caps & EFADV_DEVICE_ATTR_CAPS_RDMA_WRITE)) {
+					fprintf(stderr, "Write verb is not supported with this EFA device\n");
+					exit(1);
+				}
+#else
+				fprintf(stderr, "SRD connection not possible in WRITE verb\n");
+				exit(1);
+#endif
 			}
+
 			if (user_param->size > efa_device_attr.max_rdma_size) {
 				if (user_param->test_method == RUN_ALL || !user_param->req_size) {
 					fprintf(stderr, " Max RDMA request size is %u\n",
@@ -1946,13 +2010,13 @@ int check_mtu(struct ibv_context *context,struct perftest_parameters *user_param
 					fprintf(stderr, " Changing to this size\n");
 					user_param->size = efa_device_attr.max_rdma_size;
 				} else {
-					fprintf(stderr," Max RDMA read size in SRD cannot be greater than %u\n",
+					fprintf(stderr, " Max RDMA request size in SRD cannot be greater than %u\n",
 						efa_device_attr.max_rdma_size);
 					return FAILURE;
 				}
 			}
 #else
-			fprintf(stderr, "SRD connection not possible in READ verb\n");
+			fprintf(stderr, "SRD connection was not found\n");
 			exit(1);
 #endif
 		}
@@ -1990,7 +2054,7 @@ int rdma_cm_get_rdma_address(struct perftest_parameters *user_param,
 	char port[6] = "", error_message[ERROR_MSG_SIZE] = "";
 
 	sprintf(port, "%d", user_param->port);
-	hints->ai_family = AF_INET;
+	hints->ai_family = user_param->ai_family;
 	// if we have servername specified, it is a client, we should use server name
 	// if it is not specified, we should use explicit source_ip if possible
 	if ((NULL != user_param->servername) || (!user_param->has_source_ip)) {
@@ -2073,6 +2137,7 @@ int rdma_cm_initialize_ud_connection_parameters(struct pingpong_context *ctx,
 	struct ibv_qp_init_attr init_attr;
 	struct ibv_wc wc;
 	struct cma_node *cm_node;
+	void* grh_buffer;
 
 	needed = ((user_param->connection_type == UD)
 				&& ((user_param->tst == BW && user_param->duplex)
@@ -2097,12 +2162,27 @@ int rdma_cm_initialize_ud_connection_parameters(struct pingpong_context *ctx,
 	}
 
 	cm_node = &ctx->cma_master.nodes[connection_index];
+	grh_buffer = ctx->buf[connection_index];
+
+	if (user_param->memory_type != MEMORY_HOST){
+		struct pingpong_dest *temp_buffer;
+		ALLOCATE(temp_buffer, struct pingpong_dest, sizeof(struct pingpong_dest));
+		ctx->memory->copy_buffer_to_host(temp_buffer, ctx->buf[connection_index], sizeof(struct pingpong_dest));
+		grh_buffer = temp_buffer;
+	}
 
 	ctx->ah[connection_index] = ibv_create_ah_from_wc(ctx->pd, &wc,
-		ctx->buf[connection_index], cm_node->cma_id->port_num);
+		grh_buffer, cm_node->cma_id->port_num);
+
+	if (user_param->memory_type != MEMORY_HOST){
+		free(grh_buffer);
+	}
+
 	user_param->ah_allocated = 1;
 	ibv_query_qp(cm_node->cma_id->qp, &attr, IBV_QP_QKEY, &init_attr);
+	//coverity[uninit_use_in_call]
 	cm_node->remote_qpn = ntohl(wc.imm_data);
+	//coverity[uninit_use]
 	cm_node->remote_qkey = attr.qkey;
 	return rc;
 
@@ -2153,6 +2233,7 @@ int rdma_cm_send_ud_connection_parameters(struct pingpong_context *ctx,
 		cqes = ibv_poll_cq(ctx->send_cq, 1, &wc);
 	} while (cqes == 0);
 
+	//coverity[uninit_use]
 	rc = wc.status || wc.opcode != IBV_WC_SEND || wc.wr_id != 0;
 	if (rc) {
 		sprintf(error_message,
@@ -2278,7 +2359,7 @@ int rdma_cm_route_handler(struct pingpong_context *ctx,
 	}
 
 	ctx->cm_id = cma_id;
-	rc = create_qp_main(ctx, user_param, connection_index, 0);
+	rc = create_qp_main(ctx, user_param, connection_index);
 	if (rc) {
 		error_message = "Failed to create QP.";
 		goto error;
@@ -2315,7 +2396,8 @@ error:
 *
 ******************************************************************************/
 int rdma_cm_connection_request_handler(struct pingpong_context *ctx,
-		struct perftest_parameters *user_param, struct rdma_cm_id *cma_id)
+				       struct perftest_parameters *user_param,
+				       struct rdma_cm_event *event, struct rdma_cm_id *cma_id)
 {
 	int rc, connection_index;
 	char *error_message = "";
@@ -2345,7 +2427,7 @@ int rdma_cm_connection_request_handler(struct pingpong_context *ctx,
 	}
 
 	ctx->cm_id = cm_node->cma_id;
-	rc = create_qp_main(ctx, user_param, connection_index, 0);
+	rc = create_qp_main(ctx, user_param, connection_index);
 	if (rc) {
 		error_message = "Failed to create QP.";
 		goto error_2;
@@ -2354,8 +2436,15 @@ int rdma_cm_connection_request_handler(struct pingpong_context *ctx,
 	memset(&conn_param, 0, sizeof(conn_param));
 
 	if (user_param->verb == READ || user_param->verb == ATOMIC) {
-		conn_param.responder_resources = user_param->out_reads;
-		conn_param.initiator_depth = user_param->out_reads;
+		/* Clamp responder depth based on initiator resources on the peer */
+		conn_param.responder_resources =
+			(user_param->out_reads > event->param.conn.initiator_depth)
+			? event->param.conn.initiator_depth : user_param->out_reads;
+
+		/* Clamp initiator depth based on responder resources on the peer */
+		conn_param.initiator_depth =
+			(user_param->out_reads > event->param.conn.responder_resources)
+			? event->param.conn.responder_resources : user_param->out_reads;
 	}
 
 	conn_param.retry_count = user_param->retry_count;
@@ -2390,6 +2479,7 @@ int rdma_cm_connection_request_handler(struct pingpong_context *ctx,
 	ctx->cma_master.nodes[connection_index].connected = 1;
 	ctx->cma_master.connection_index++;
 	ctx->cma_master.connects_left--;
+	ctx->cma_master.disconnects_left++;
 	return rc;
 
 error_2:
@@ -2467,7 +2557,7 @@ int rdma_cm_events_dispatcher(struct pingpong_context *ctx,
 		rc = rdma_cm_route_handler(ctx, user_param, cma_id);
 		break;
 	case RDMA_CM_EVENT_CONNECT_REQUEST:
-		rc = rdma_cm_connection_request_handler(ctx, user_param, cma_id);
+		rc = rdma_cm_connection_request_handler(ctx, user_param, event, cma_id);
 		break;
 	case RDMA_CM_EVENT_ESTABLISHED:
 		rc = rdma_cm_connection_established_handler(ctx, user_param, event);
@@ -2509,6 +2599,7 @@ int rdma_cm_connect_events(struct pingpong_context *ctx,
 			goto error;
 		}
 
+		//coverity[uninit_use]
 		rc = rdma_cm_events_dispatcher(ctx, user_param, event->id, event);
 		if (rc) {
 			error_message = "Failed to handle RDMA CM event.";
@@ -2561,6 +2652,7 @@ int rdma_cm_disconnect_nodes(struct pingpong_context *ctx,
 			error_message = "Failed to get RDMA CM event.";
 			goto error;
 		}
+		//coverity[uninit_use]
 		rc = rdma_cm_events_dispatcher(ctx, user_param, event->id, event);
 		if (rc) {
 			error_message = "Failed to handle RDMA CM event.";
@@ -2633,6 +2725,7 @@ int rdma_cm_server_connection(struct pingpong_context *ctx,
 	return rc;
 
 destroy_id:
+	//coverity[uninit_use_in_call]
 	rdma_destroy_id(listen_id);
 
 error:
@@ -2749,6 +2842,7 @@ int create_rdma_cm_connection(struct pingpong_context *ctx,
 
 	if (rc) {
 		error_message = "Failed to create RDMA CM connection.";
+		free(hints.ai_src_addr);
 		goto destroy_event_channel;
 	}
 
@@ -2759,6 +2853,8 @@ int create_rdma_cm_connection(struct pingpong_context *ctx,
 		goto destroy_rdma_id;
 	}
 
+	free(hints.ai_src_addr);
+
 	return rc;
 
 
@@ -2768,6 +2864,7 @@ destroy_rdma_id:
 			rdma_destroy_id(ctx->cma_master.nodes[i].cma_id);
 	}
 	free(ctx->cma_master.nodes);
+	free(hints.ai_src_addr);
 
 destroy_event_channel:
 	rdma_destroy_event_channel(ctx->cma_master.channel);
