@@ -61,18 +61,12 @@
 #endif
 #include "get_clock.h"
 #include "perftest_counters.h"
+#include "memory.h"
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#ifdef HAVE_CUDA
-#include CUDA_PATH
-#endif
-
-#ifdef HAVE_ROCM
-#include <hip/hip_runtime_api.h>
-#endif
 
 /* Connection types available. */
 #define RC  (0)
@@ -162,7 +156,7 @@
 #define MAX_HOP_LIMIT	(255)
 #define MIN_IB_PORT   (1)
 #define MIN_ITER      (5)
-#define MAX_ITER      (100000000)
+#define MAX_ITER      (4294967296)
 #define MIN_TX 	      (1)
 #define MAX_TX	      (15000)
 #define MIN_SL	      (0)
@@ -178,8 +172,8 @@
 #define UC_MAX_RX     (16000)
 #define MIN_CQ_MOD    (1)
 #define MAX_CQ_MOD    (1024)
-#define MAX_INLINE    (912)
-#define MAX_INLINE_UD (884)
+#define MAX_INLINE    (1024)
+#define MAX_INLINE_UD (1024)
 #define MIN_EQ_NUM    (0)
 #define MAX_EQ_NUM    (2048)
 
@@ -283,7 +277,7 @@ t_stdev: %.2f,\npercentile_99: %.2f,\npercentile_99.9: %.2f,\n"
 #define CHECK_VALUE_NON_NEGATIVE(arg,type,name,not_int_ptr) {\
 	CHECK_VALUE(arg,type,name,not_int_ptr)\
 	if (arg < 0) {\
-		fprintf(stderr," %s must be non-negative\n",name);\
+		fprintf(stderr," %s must be non-negative %s\n",name,#type);\
 		return 1;\
 	}\
 }
@@ -321,6 +315,13 @@ t_stdev: %.2f,\npercentile_99: %.2f,\npercentile_99.9: %.2f,\n"
 
 #define GET_ARRAY_SIZE(arr) (sizeof((arr)) / sizeof((arr[0])))
 
+#define ASSERT(x) \
+	do { \
+	if (!(x)) { \
+		fprintf(stdout, "Assertion \"%s\" failed at %s:%d\n", #x, __FILE__, __LINE__); \
+	} \
+} while (0)
+
 /* The Verb of the benchmark. */
 typedef enum { SEND , WRITE, READ, ATOMIC } VerbType;
 
@@ -373,13 +374,17 @@ enum ctx_device {
 	MLX5GENVF		= 19,
 	BLUEFIELD		= 20,
 	BLUEFIELD2		= 21,
-	INTEL_ALL		= 22,
+	INTEL_GEN1		= 22,
 	NETXTREME		= 23,
 	EFA			= 24,
 	CONNECTX6LX		= 25,
 	CONNECTX7		= 26,
 	QLOGIC_AHP		= 27,
 	BLUEFIELD3		= 28,
+	ERDMA			= 29,
+	HNS			= 30,
+	CONNECTX8		= 31,
+  INTEL_GEN2		= 32,
 };
 
 /* Units for rate limiter */
@@ -413,6 +418,7 @@ struct check_alive_data {
  *	 	src_mac - source MAC address of the packet
  *	 	dst_mac - destination MAC address of the packet
  *	 	eth_type - IP/or size of ptk
+ *		user_param - the parameters element.
  *
  *
 struct ETH_header {
@@ -428,6 +434,15 @@ struct ETH_vlan_header {
         uint16_t eth_type;
 }__attribute__((packed));*/
 
+enum memory_type {
+	MEMORY_HOST,
+	MEMORY_MMAP,
+	MEMORY_CUDA,
+	MEMORY_ROCM,
+	MEMORY_NEURON,
+	MEMORY_HL
+};
+
 struct perftest_parameters {
 
 	int				port;
@@ -489,12 +504,12 @@ struct perftest_parameters {
 	int				is_ethertype;
 	int				cpu_freq_f;
 	int				connection_type;
+	int				use_null_mr;
 	int				num_of_qps;
 	int				use_event;
 	int				eq_num;
 	int				use_eq_num;
 	int 				inline_size;
-	int				inline_recv_size;
 	int				out_reads;
 	int				rx_depth;
 	int				duplex;
@@ -543,15 +558,18 @@ struct perftest_parameters {
 	int				buff_size;
 	int             		pkey_index;
 	int				raw_qos;
-#ifdef HAVE_CUDA
-	int				use_cuda;
+	int				has_payload_modification;
+	char*				payload_file_path;
+	char*				payload_content;
+	int				payload_length;
+	enum memory_type		memory_type;
+	struct memory_ctx		*(*memory_create)(struct perftest_parameters *params);
 	int				cuda_device_id;
 	char				*cuda_device_bus_id;
-#endif
-#ifdef HAVE_ROCM
-	int				use_rocm;
+	int				use_cuda_dmabuf;
 	int				rocm_device_id;
-#endif
+	int				neuron_core_id;
+	char				*hl_device_bus_id;
 	char				*mmap_file;
 	unsigned long			mmap_offset;
 	/* New test params format pilot. will be used in all flags soon,. */
@@ -584,6 +602,7 @@ struct perftest_parameters {
 	int 				dont_xchg_versions;
 	int 				ipv6;
 	int 				raw_ipv6;
+	int 				ai_family;
 	int 				report_per_port;
 	int 				use_odp;
 	int				use_hugepages;
@@ -595,7 +614,6 @@ struct perftest_parameters {
 	int				masked_atomics;
 	int				cycle_buffer;
 	int				cache_line_size;
-	int				use_res_domain;
 	int				mr_per_qp;
 	uint16_t			dlid;
 	uint8_t				traffic_class;
@@ -607,7 +625,7 @@ struct perftest_parameters {
 	int				use_ooo;
 	int				vlan_en;
 	uint32_t			vlan_pcp;
-	void 				(*print_eth_func)(void*);
+	void 				(*print_eth_func)(void*, struct perftest_parameters*, struct memory_ctx*);
 	int				disable_pcir;
 	struct counter_context		*counter_ctx;
 	char				*source_ip;
@@ -701,9 +719,9 @@ const int str_link_layer(const char *str);
  */
 int parser(struct perftest_parameters *user_param,char *argv[], int argc);
 
-/* check_link_and_mtu
+/* check_link
  *
- * Description : Configures test MTU,inline and link layer of the test.
+ * Description : Configures test inline and link layer of the test.
  *
  * Parameters :
  *
